#############
# $Id: bam_markduplicate.pm 14173 2011-09-13 09:45:29Z ajb $
# Created By: gq1
# Last Maintained By: $Author: ajb $
# Created On: 2010-06-25
# Last Changed On: $Date: 2011-09-13 10:45:29 +0100 (Tue, 13 Sep 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-30.0/lib/npg_pipeline/archive/file/generation/bam_markduplicate.pm $

package npg_pipeline::archive::file::generation::bam_markduplicate;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly;
use List::MoreUtils qw{any};

extends q{npg_pipeline::archive::file::generation};

with qw{npg_common::roles::run::lane::tag_info};

Readonly::Scalar our $VERSION  => do { my ($r) = q$LastChangedRevision: 14173 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $DEFAULT_RESOURCES => q{-R 'select[mem>8000] rusage[mem=8000]' -M8000000};
Readonly::Scalar our $JAVA_MACHINE_MEMORY => q{--java_Xmx 7000};

sub generate {
  my ( $self, $arg_refs ) = @_;

  my $id_run = $self->id_run();

  $self->log(qq{Creating Jobs to run MarkDuplicates for bam files for run $id_run});

  my @positions = $self->positions( $arg_refs );

  my @job_ids;

  my $href = $self->lanes_by_split_nonsplit( @positions );

  my @bsub_commands;

  if ( scalar @{ $href->{split_nc_lanes} } ) {
    push @bsub_commands, $self->_generate_bsub_commands( {
      positions => $href->{split_nc_lanes},
      indexed_positions => $href->{indexed_lanes},
      multiplex => 0,
      split_nc => 1,
      arg_refs => $arg_refs,
    } );
  }

  if ( scalar @{ $href->{non_split_nc_lanes} } ) {
    push @bsub_commands, $self->_generate_bsub_commands( {
      positions => $href->{non_split_nc_lanes},
      indexed_positions => $href->{indexed_lanes},
      multiplex => 0,
      split_nc => 0,
      arg_refs => $arg_refs,
    } );
  }

  if ( scalar @{ $href->{spiked_phix_lanes} } ) {

    push @bsub_commands, $self->_generate_bsub_commands( {
      positions => $href->{spiked_phix_lanes},
      indexed_positions => $href->{indexed_lanes},
      multiplex => 0,
      split_nc => 0,
      spiked_phix => 1,
      arg_refs => $arg_refs,
    } );
  }

  if ( scalar @{ $href->{split_nc_mp_lanes} } ) {
    push @bsub_commands, $self->_generate_bsub_commands( {
      positions => $href->{split_nc_mp_lanes},
      multiplex => 1,
      split_nc => 1,
      arg_refs => $arg_refs,
    } );
  }

  if ( scalar @{ $href->{non_split_nc_mp_lanes} } ) {
    push @bsub_commands, $self->_generate_bsub_commands( {
      positions => $href->{non_split_nc_mp_lanes},
      multiplex => 1,
      split_nc => 0,
      arg_refs => $arg_refs,
    } );
  }

  foreach my $cmd ( @bsub_commands ) {
    push @job_ids, $self->submit_bsub_command( $cmd );
  }


  return @job_ids;
}


###############
# responsible for generating the bsub command to be executed for one bam file
sub _generate_bsub_commands {
  my ( $self, $outer_arg_refs ) = @_;

  my $arg_refs = $outer_arg_refs->{arg_refs};

  my $positions = $outer_arg_refs->{positions};
  my $multiplex = $outer_arg_refs->{multiplex};
  my $split_nc  = $outer_arg_refs->{split_nc};
  my $spiked_phix  = $outer_arg_refs->{spiked_phix};

  my $fastq_files             = $arg_refs->{fastq_files};
  my $required_job_completion = $arg_refs->{required_job_completion};

  $self->clear_lane_tag_pairs();

  my $id_run                  = $self->id_run();

  my $output_path = $self->archive_path() . q{/};

  my $lsf_array_string;
  my $full_bam_name = $id_run . q{_};

  if ( $multiplex ) {

    foreach my $position ( @{ $positions } ) {
      $self->add_lane_tag_pairs( $position );
    }

    $lsf_array_string = $self->lsf_job_array_from_lane_tag_pairs();
    $output_path .= q{lane} . $self->position_decode_string() . q{/};
    $full_bam_name .=  $self->position_decode_string();

  } else {

    my @wanted_positions;

    if ( $spiked_phix ) {
      @wanted_positions = @{ $positions };
    } else {
      foreach my $position ( @{ $positions } ) {
        if ( ! ( any { $position == $_ } @{ $outer_arg_refs->{indexed_positions} } ) ) {
          push @wanted_positions, $position;
        }

      }
    }

    # if there are no wanted positions, since they are all indexed lanes, then return an empty array, as no bam_markduplicates need running
    if ( ! scalar @wanted_positions ) {
      return ();
    }
    $lsf_array_string = $self->create_array_string( @wanted_positions );
    $full_bam_name .= $self->lsb_jobindex();

  }

  my $script_bin = $self->internal_script_bin();

  my $log_folder = $self->make_log_dir( $self->archive_path() );

  my $outfile = $log_folder . q{/} . $self->bsub_output_file();

  my $job_name = $self->external_script_names_conf()->{bam_markduplicate_script} . q{_} . $id_run . q{_} . $self->timestamp();

  my $out_subs_name = $job_name . q{.%I.%J};
  $outfile =~ s/substitute_this/$out_subs_name/xms;

  $job_name .= $lsf_array_string;
  if ( ! $self->no_array_cpu_limit() ) {
    $job_name .= q{%} . $self->array_cpu_limit();
  }
  $job_name = q{'} . $job_name . q{'};

  my @types;
  if ( $split_nc ) {
    @types = ( qw(human nonhuman) );
  }elsif( $spiked_phix ) {
    @types = ( qw{phix} );
  } else {
    @types = ( qw{none} );
  }

  my @bsub_commands;

#  $full_bam_name = $output_path . $full_bam_name;

  foreach my $type ( @types ) {

    my $specific_bam_filename = $full_bam_name;

    if ( $type ne q{none} ) {
      $specific_bam_filename .= q{_} . $type;
    }

    if ( $multiplex ) {
      $specific_bam_filename .= q{#} . $self->tag_index_decode_string();
    }


    my $job_command = qq{$script_bin/} . $self->external_script_names_conf()->{bam_markduplicate_script} . qq{ $JAVA_MACHINE_MEMORY --replace_file 1 --id_run $id_run --position };
    if ( $multiplex ) {
      $job_command .= $self->position_decode_string();
      $job_command .= q{ --tag_index } . $self->tag_index_decode_string();
    } else {
      $job_command .= $self->lsb_jobindex();
    }

    $job_command .= q{ --input_bam }     . $output_path . $specific_bam_filename . q{.bam};
    $job_command .= q{ --output_bam }    . $output_path . $specific_bam_filename . q{_mk.bam};
    $job_command .= q{ --metrics_json }  . $output_path . q{qc/} . $specific_bam_filename . q{_bam_flagstats.json};

    if ( $type ne q{none} ) {
      $job_command .= qq{ --human_split $type};
    }

    my $loop_job_name = $job_name;
    my $loop_outfile  = $outfile;

    if ( $type ne q{none} ) {
      $loop_job_name =~ s/_{$id_run}_/_{$id_run}_{$type}_/xms;
      $loop_outfile  =~ s/_{$id_run}_/_{$id_run}_{$type}_/xms;
    }

    if ( $multiplex ) {
      $loop_job_name =~ s/_{$id_run}_/_{$id_run}_mp_/xms;
      $loop_outfile  =~ s/_{$id_run}_/_{$id_run}_mp_/xms;
    }

    my $resources = ( $self->fs_resource_string( {
      counter_slots_per_job => 4,
      resource_string => $DEFAULT_RESOURCES,
    } ) );

    my $job_sub = q{bsub -q } . $self->lsf_queue() . qq{ $resources $required_job_completion -J $loop_job_name -o $loop_outfile '$job_command'};

    if ( $self->verbose() ) { $self->log($job_sub); }

    push @bsub_commands, $job_sub;
  }

  return @bsub_commands;
}


no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::generation::bam_markduplicate

=head1 VERSION

$LastChangedRevision: 14173 $

=head1 SYNOPSIS

  my $oAfgfq = npg_pipeline::archive::file::generation::bam->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating bam file.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for creating the fastq files, returning an array of job_ids.

  my @job_ids = $oAfgfq->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    solexa_bin               => q{/software/solexa/bin};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
