#############
# $Id: dispatch_tree.pm 14173 2011-09-13 09:45:29Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-08-07
# Last Changed On: $Date: 2011-09-13 10:45:29 +0100 (Tue, 13 Sep 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-30.0/lib/npg_pipeline/dispatch_tree.pm $

package npg_pipeline::dispatch_tree;
use Moose;
use JSON;
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 14173 $ =~ /(\d+)/mxs; $r; };

extends q{npg_pipeline::base};
with q{npg_common::roles::run::path_info};

has q{script_name}    => (isa => q{Str},      is => q{ro}, predicate => q{has_script_name}, writer => q{_set_script_name});
has q{schema}         => (isa => q{Str},      is => q{ro}, predicate => q{has_schema},      writer => q{_set_schema});
has q{run_folder}     => (isa => q{Str},      is => q{ro}, predicate => q{has_run_folder},  writer => q{_set_run_folder});
has q{timestamp}      => (isa => q{Str},      is => q{ro}, predicate => q{has_timestamp},   writer => q{_set_timestamp});
has q{json_structure} => (isa => q{Str},      is => q{ro}, predicate => q{has_json_structure}, writer => q{_set_json_structure});
has q{functions}      => (isa => q{ArrayRef}, is => q{ro}, lazy_build => 1, writer => q{_set_functions});

sub BUILD {
  my ($self) = @_;
  if ($self->has_json_structure()) {
    $self->_populate_with_json();
  }
  return 1;
}

sub _build_functions {
  my ($self) = @_;
  return [];
}

sub append_to_functions {
  my ($self, $function_info) = @_;
  push @{$self->functions()}, $function_info;
  return 1;
}

sub _data_structure {
  my ($self) = @_;
  my $href = {};

  if ($self->has_script_name()) {
    $href->{script_name} = $self->script_name();
  }
  if ($self->has_schema()) {
    $href->{schema} = $self->schema();
  }
  if ($self->has_run_folder()) {
    $href->{run_folder} = $self->run_folder();
  }
  if ($self->has_id_run()) {
    $href->{id_run} = $self->id_run();
  }
  if ($self->has_timestamp()) {
    $href->{timestamp} = $self->timestamp();
  }

  $href->{functions} = $self->functions();

  return $href;
}

sub tree_as_json {
  my ($self) = @_;
  my $href = $self->_data_structure();
  $self->_set_json_structure(to_json($href));
  return $self->json_structure();
}

sub _populate_with_json {
  my ($self) = @_;
  my $href = from_json($self->json_structure());
  foreach my $key (keys %{$href}) {
    my $writer_method = q{_set_}.$key;
    $self->$writer_method($href->{$key});
  }
  return $self;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::dispatch_tree

=head1 VERSION

$LastChangedRevision: 14173 $

=head1 SYNOPSIS

  my $oTree = npg_pipeline::dispatch_tree->new({
    script_name => <script_name>,
    schema      => <schema_name>,
    run_folder  => <run_folder>,
    id_run      => <id_run>,
    timestamp   => <timestamp>,
  });

All the above are optional parameters that you might want to use in the data structure that will be stored

The object can also be given the json structure it would create, and populate itself with that

  my $oTree = npg_pipeline::dispatch_tree->new({
    json_structure => $json_string,
  });

=head1 DESCRIPTION

Object model for storing, creating and reading a dispatch tree from a pipeline

=head1 SUBROUTINES/METHODS

=head2 BUILD - sets up the functions array on new construction

=head2 append_to_functions - appends into the data structure an element which needs to be stored for the functions called in order

  $oTree->append_to_functions({
    function => <function_name>,
    job_ids_launched => [job_id_1,job_id_2,...],
    job_dependencies => q{-w'done(1) && done(2) && done(3)'},
  });

=head2 tree_as_json - outputs the stored data tree/structure as a JSON string

  my $sJsonStructure = $oTree->tree_as_json();

=head2 functions - returns arrayref of the functions or function information stored in the order appended

  my $aFunctions = $oTree->functions();

=head2 schema - accessor to obtain the schema name used
=head2 script_name - accessor to obtain the script_name used
=head2 run_folder - accessor to obtain the run_folder
=head2 id_run - accessor to obtain the run id
=head2 timestamp - accessor to obtain the timestamp

  my $string = $oTree->accessor_name();

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item JSON

=item npg_common::roles::run::path_info

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
