#############
# $Id: PB_cal.pm 14173 2011-09-13 09:45:29Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2010-07-14
# Last Changed On: $Date: 2011-09-13 10:45:29 +0100 (Tue, 13 Sep 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-30.0/lib/npg_pipeline/pluggable/harold/PB_cal.pm $

package npg_pipeline::pluggable::harold::PB_cal;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 14173 $ =~ /(\d+)/mxs; $r; };

extends qw{npg_pipeline::pluggable::harold};
with q{npg_common::roles::run::fs_resource};
with q{npg_pipeline::roles::business::bustard_lsf_reqs};

=head1 NAME

npg_pipeline::pluggable::harold::PB_cal

=head1 VERSION

$LastChangedRevision: 14173 $

=head1 SYNOPSIS

  my $oHaroldBustard = npg_pipeline::pluggable::harold::PB_cal->new({
    run_folder => $sRunFolder,
  })

=head1 DESCRIPTION

pluggable module runner for the PB_cal pipeline

=head1 SUBROUTINES/METHODS

=head2 BUILD

ensures that the folders are built that everything expects. If they are provided, it will use them, else it will create them

Notes:

1) intensity_path - there is a bug in the imported path_info role which needs fixing. As such, at this time, if intensity_path isn't
directly specified, then it will take runfolder_path and add /Data/Intensities

2) if recalibrated_path, pb_cal_path, bustard_path and basecall_path are specified, it will use these (and map pb_cal_path <->
recalibrated_path and bustard_path <-> basecall_path ), else it will create based on whether no_recalibration is set, and utilise
the timestamp

3) This is designed to be launched after the harold.pm BUILD method is called as per Moose v1.15. If this should be altered in
a future version of Moose, the this may break

=cut

sub BUILD {
  my ( $self ) = @_;

  if ( ! $self->has_intensity_path() ) {
    $self->_set_intensity_path( $self->runfolder_path() . q{/Data/Intensities} );
  }

  my $timestamp = $self->timestamp();

  my $pathways = {
    recalibrated_path => undef,
    pb_cal_path => undef,
    bustard_path => undef,
    basecall_path => undef,
  };

  # for each of the paths, see if they have been prepopulated
  foreach my $path ( keys %{ $pathways } ) {
    my $has_method = q{has_} . $path;
    if ( $self->$has_method() ) {
      $pathways->{$path} = $self->$path();
    }
  }

  # if bustard_path or basecall_path are not set, but the other is, match them up
  if ( $pathways->{bustard_path} && ! $pathways->{basecall_path} ) {
    $pathways->{basecall_path} = $pathways->{bustard_path};
  }
  if ( ! $pathways->{bustard_path} && $pathways->{basecall_path} ) {
    $pathways->{bustard_path} = $pathways->{basecall_path};
  }

  # if recalibrated_path or pb_cal_path are not set, but the other is, match them up
  if ( $pathways->{recalibrated_path} && ! $pathways->{pb_cal_path} ) {
    $pathways->{pb_cal_path} = $pathways->{recalibrated_path};
  }
  if ( ! $pathways->{recalibrated_path} && $pathways->{pb_cal_path} ) {
    $pathways->{recalibrated_path} = $pathways->{pb_cal_path};
  }

  # if there is no bustard_path and basecall_path, then create them and store
  if ( ! $pathways->{bustard_path} ) {
    my $bustard_level_dir = q{PB_basecalls_} . $timestamp;
    $self->make_log_dir( $self->intensity_path() . q{/} . $bustard_level_dir );
    $pathways->{bustard_path}  = $self->intensity_path() . q{/} . $bustard_level_dir;
    $pathways->{basecall_path} = $self->intensity_path() . q{/} . $bustard_level_dir;
  }

  # if there is no recalibrated_path and pb_cal_path, then create them and store
  if ( ! $pathways->{recalibrated_path} ) {
    my $recalibrated_level_dir = $self->no_recalibration() ? q{no_cal}
                               :                             q{PB_cal}
                               ;
    $self->make_log_dir( $pathways->{bustard_path} . q{/} . $recalibrated_level_dir );
    $pathways->{recalibrated_path} = $pathways->{bustard_path} . q{/} . $recalibrated_level_dir;
    $pathways->{pb_cal_path}       = $pathways->{bustard_path} . q{/} . $recalibrated_level_dir;
  }
  # for each of these, go and set them (we know we must have created them by now)
  foreach my $path ( keys %{ $pathways } ) {
    my $set_method = q{_set_} . $path;
    $self->$set_method( $pathways->{$path} );
  }

  return 1;
}

=head2 make_multiplex_lane_specific_tag_files

This doesn't need to be submitted as a bsub job at this time.
It will create lane specific tag files, so that these can be used for index determination in preference
to a default containing all tags (unless a tag file has been provided, in which case, that will be used)

=cut

sub make_multiplex_lane_specific_tag_files {     # TODO: there's another copy of this in harold/bustard.pm
  my ($self, @args) = @_;

  my $o_create_lane_tag_file = $self->new_with_cloned_attributes(q{npg_pipeline::analysis::create_lane_tag_file});

  eval {
    $o_create_lane_tag_file->generate();
    1;
  } or do {
    croak q{Problem creating lane specific tag file: } . $EVAL_ERROR;
  };

  return ();
}

=head2 check_and_fix_stats_files

This should not be submitted as a bsub job at this time

=cut

sub check_and_fix_stats_files {
  my ( $self, @args ) = @_;

  my $checker = $self->new_with_cloned_attributes( q{npg_pipeline::analysis::CheckStats} );

  eval {
    $checker->check_and_fix_stats();
    1;
  } or do {
    croak q{Broken files: } . $EVAL_ERROR;
  };

  return ();
}

=head2 run_bcl_setup

Run the Illumina pipeline script to create a makefile in the runfolder
Basecalls directory for creating qseq files from mirrored .bcl files.

=cut

sub run_bcl_setup {
  my ($self, @args) = @_;

  return $self->_run_harold_steps( q{run_bcl_setup}, @args);
}

=head2 make_qseq_files

Having created a make file in the run_bcl_setup step, submit the make.

=cut

sub make_qseq_files {
  my ($self, @args) = @_;

  return $self->_run_harold_steps( q{make_qseq_files}, @args);
}

=head2 demultiplex

If the run is indexed/multiplexed, then sort files out to ensure that GERALD (for 1.6) can cope.

=cut

sub demultiplex {     # TODO: there's another copy of this in harold/bustard.pm
  my ($self, @args) = @_;

  my $required_job_completion = shift @args;
  my $multiplex = $self->new_with_cloned_attributes(q{npg_pipeline::analysis::multiplex});

  my @job_ids = $multiplex->submit_index_decoder_jobs({
    required_job_completion => $required_job_completion,
  });

  return @job_ids;
}

=head2 harold_alignment_files

Generate the alignment files to now be used for generating calibration tables

=cut

sub harold_alignment_files {
  my ($self, @args) = @_;

  if ( $self->no_recalibration() ) {
    $self->log( q{no_recalibration declared, no recalibration will be performed} );
    return ();
  }

  return $self->_run_harold_steps( q{generate_alignment_files}, @args);
}

=head2 harold_calibration_tables

Generate the calibration tables used for harold recalibration

=cut

sub harold_calibration_tables {
  my ($self, @args) = @_;

  if ( $self->no_recalibration() ) {
    $self->log( q{no_recalibration declared, no recalibration will be performed} );
    return ();
  }

  return $self->_run_harold_steps( q{generate_calibration_table}, @args);
}

=head2 harold_recalibration

submit the recalibration jobs

=cut

sub harold_recalibration {
  my ($self, @args) = @_;

  if ( $self->no_recalibration() ) {
    $self->log( q{no_recalibration declared, no recalibration will be performed} );
    return ();
  }

  return $self->_run_harold_steps( q{generate_recalibrated_qseqs}, @args);
}

sub _run_harold_steps {
  my ($self, $method, @args) = @_;

  my $required_job_completion = shift @args;

  my $harold = $self->new_with_cloned_attributes(q{npg_pipeline::analysis::harold_calibration});

  my @job_ids = $harold->$method({
    required_job_completion => $required_job_completion,
  });

  return @job_ids;
}


=head2 run_post_qseq

Run the script which acts on files and produces the archival files and does internal qc (post_qseq)

=cut

sub run_post_qseq {

  my ($self, @args) = @_;

  # check to see if cleanup/post_qseq is has been turned off
  if ($self->no_post_qseq()) {
    $self->log(q{Post_QSEQ (cleanup/archival file creation) turned off});
    return ();
  }

  my $required_job_completion = shift @args;

  my $harold = $self->new_with_cloned_attributes(q{npg_pipeline::analysis::harold_calibration});

  my @job_ids = $harold->submit_post_qseq({
    required_job_completion => $required_job_completion,
  });

  return @job_ids;

}

=head2 generate_second_basecalls

=cut

sub generate_second_basecalls {
  my ( $self, @args ) = @_;

  if ( $self->no_secondcall() ) {
    $self->log( q{no_secondcall specified on command line, so none will be generated} );
    return ();
  }

  my $required_job_completion = shift @args;

  my $second_basecall = $self->new_with_cloned_attributes( q{npg_pipeline::analysis::SecondBaseCall} );

  my @job_ids = $second_basecall->submit_to_lsf( {
    required_job_completion => $required_job_completion,
  } );

  return @job_ids;
}

sub _default_function_order {
  my ($self) = @_;
  return $self->function_order_conf()->{pb_cal};
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
