#############
# $Id: post_qseq.pm 14173 2011-09-13 09:45:29Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-11-05
# Last Changed On: $Date: 2011-09-13 10:45:29 +0100 (Tue, 13 Sep 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-30.0/lib/npg_pipeline/pluggable/harold/post_qseq.pm $

package npg_pipeline::pluggable::harold::post_qseq;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 14173 $ =~ /(\d+)/mxs; $r; };

extends qw{npg_pipeline::pluggable::harold};
with qw{npg_pipeline::roles::business::file_constructs};

# this provides an in-built function order for post-qseq should none be provided

sub _default_function_order {
  my ($self) = @_;
  return $self->function_order_conf()->{post_qseq};
}

=head1 NAME

npg_pipeline::pluggable::harold::post_qseq

=head1 VERSION

$LastChangedRevision: 14173 $

=head1 SYNOPSIS

  my $oPostQseq = npg_pipeline::pluggable::harold::post_qseq->new();

=head1 DESCRIPTION

Pluggable pipeline module for the post_qseq pipeline

=head1 SUBROUTINES/METHODS

=head2 create_archive_directory

creates the archive and qc directories. requires a Latest Summary link to be present. probably best done just prior to reate_fastq, although could be earlier

=cut

sub create_archive_directory {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my $afg = $self->new_with_cloned_attributes(q{npg_pipeline::archive::folder::generation});
  eval {
    $afg->create_dir();
    1;
  } or do {
    croak $EVAL_ERROR;
  };
  return ();
}

=head2 touch_mp_fastq_files

ensures that there will be a fastq file for every tag for a lane, in case there are no PF reads found for a tag

=cut

sub touch_mp_fastq_files {
  my ( $self, @args ) = @_;

  my $required_job_completion = shift @args;
  my $afg = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::generation::touchSplitFiles});
  return $afg->generate();
}

=head2 create_psuedo_qseq_custom

some runs (or just lanes) won't have been recalibrated, so ensure that there is a qseq_custom file for each qseq file, if not, create a softlink to create one

=cut

sub create_psuedo_qseq_custom {
  my ($self, @args) = @_;
  # this will be fast, and needs to be run before any jobs which need submitting, after the Latest_Summary link has been created 
  eval {
    my $cpqc = $self->new_with_cloned_attributes(q{npg_pipeline::analysis::create_psuedo_qseq_custom});
    $cpqc->run_create();
  } or do {
    croak $EVAL_ERROR;
  };
  return ();
}

=head2 correct_qseq_for_index_runs

if the run is indexed, then the run will need indexed qseq_custom files to be generated

=cut

sub correct_qseq_for_index_runs {
  my ($self, @args) = @_;

  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = shift @args;

  my $cqseq = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::correct_qseq});
  my @jids = $cqseq->correct($arg_refs);

  return @jids;
}

=head2 create_fastq

fastq archive file creation
requires create_archive_directory

=cut

sub create_fastq {
  my ($self, @args) = @_;
  return $self->_archival_file_creation(q{fastq}, @args);
}

=head2 create_fastqcheck

fastqcheck archive file creation
requires create_fastq

=cut

sub create_fastqcheck {
  my ($self, @args) = @_;
  return $self->_archival_file_creation(q{fastqcheck}, @args);
}

=head2 create_fastqcheck_by_tag

fastqcheck by tag file creation
if it is multiplex run 
and fastq file already been split by tag

=cut

sub create_fastqcheck_by_tag{
  my ($self, @args) = @_;

  return $self->_archival_file_creation(q{fastqcheck_by_tag}, @args);
}

=head2 create_md5

md5 archive file creation
requires create_fastq, create_fastqcheck

=cut

sub create_md5 {
  my ($self, @args) = @_;
  return $self->_archival_file_creation(q{md5}, @args);
}

=head2 split_fastq_by_tag

split fastq file by tag if multiplex run

=cut

sub split_fastq_by_tag {
  my ($self, @args) = @_;
  if ($self->no_split_fastq_by_tag()) {
    $self->log(q{Skipping spliting fastq by tag});
    return ();
  }
  return $self->_archival_file_creation(q{fastq_by_tag}, @args);
}

=head2 bam_generation

bam alignment file creation

=cut

sub bam_generation {
  my ($self, @args) = @_;
  if ($self->no_bam_generation()) {
    $self->log(q{Skipping bam file generation});
    return ();
  }
  return $self->_archival_file_creation(q{bam}, @args);
}

=head2 bam_markduplicate

Picard bam MarkDuplicates

=cut

sub bam_markduplicate {
  my ($self, @args) = @_;

  return $self->_archival_file_creation(q{bam_markduplicate}, @args);
}

=head2 bam_index

Picard build bam index file

=cut

sub bam_index {
  my ($self, @args) = @_;

  return $self->_archival_file_creation(q{bam_index}, @args);
}

# refactor of the common way that archival file creation modules are named and run
sub _archival_file_creation {
  my ($self, $type, @args) = @_;
  my $arg_refs = {};

  if($type eq q{fastqcheck_by_tag}){
    $type = q{fastqcheck};
    $arg_refs->{fastqcheck_by_tag} = 1;
  }
  $arg_refs->{required_job_completion}  = shift @args;

  my $generator = $self->new_with_cloned_attributes(qq{npg_pipeline::archive::file::generation::$type});
  my @jids = $generator->generate($arg_refs);

  return @jids;

}

=head2 split_nonconsented_sequence

after initial archival files have been created, remove any unconsented human contamination

=cut

sub split_nonconsented_sequence {
  my ($self, @args) = @_;

  my $arg_refs = {
    required_job_completion => shift @args,
  };

  my $split_nc = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::split_nonconsented});
  my @jids = $split_nc->split_run_lanes($arg_refs);

  return @jids;
}

=head2 split_nonconsented_ifnotdoneyet

after initial archival files have been created, remove any unconsented human contamination
if this has not been done yet

=cut

sub split_nonconsented_ifnotdoneyet {
  my ($self, @args) = @_;

  my $arg_refs = {
    required_job_completion => shift @args,
  };

  my $split_nc = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::split_nonconsented});
  my @jids = $split_nc->split_run_lanes($arg_refs, 1);

  return @jids;
}

=head2 split_phix

after initial archival files have been created, remove any spiked phix 

=cut

sub split_phix {
  my ($self, @args) = @_;

  my $arg_refs = {
    required_job_completion => shift @args,
  };

  my $split_phix = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::split_phix});
  my @jids = $split_phix->split_run_lanes($arg_refs);

  return @jids;
}

=head2 qc_cache_reads

for qc tests, cache X reads out of the fastq file, so that the qc's can save time

=cut

sub qc_cache_reads {
  my ($self, @args) = @_;
  return $self->_qc_runner(q{cache}, @args);
}

=head2 qc_qX_yield

run the qX-yield qc test

=cut

sub qc_qX_yield { ## no critic (NamingConventions)
  my ($self, @args) = @_;
  return $self->_qc_runner(q{qX_yield}, @args);
}

=head2 qc_insert_size

run the inset_size qc test

=cut

sub qc_insert_size {
  my ($self, @args) = @_;
  return $self->_qc_runner(q{insert_size}, @args);
}

=head2 qc_adapter

run the adapter qc test

=cut

sub qc_adapter {
  my ($self, @args) = @_;
  return $self->_qc_runner(q{adapter}, @args);
}

=head2 qc_contamination

run the adapter qc test

=cut

sub qc_contamination {
  my ($self, @args) = @_;
  return $self->_qc_runner(q{contamination}, @args);
}

=head2 qc_ref_match

run the ref_match qc test

=cut

sub qc_ref_match {
  my ($self, @args) = @_;
  return $self->_qc_runner(q{ref_match}, @args);
}

=head2 qc_sequence_error

run the sequence_error test

=cut

sub qc_sequence_error {
  my ($self, @args) = @_;
  return $self->_qc_runner(q{sequence_error}, @args);
}

=head2 qc_sequence_error4spiked_phix

run the sequence_error test for teh spiked phix part

=cut

sub qc_sequence_error4spiked_phix {
  my ($self, @args) = @_;
  push @args, {autoqc_data => {spiked_phix_sequence => q[--species PhiX --sequence_type phix],},};
  return $self->_qc_runner(q{sequence_error}, @args);
}

=head2 qc_gc_fraction

run the gc_fraction test

=cut

sub qc_gc_fraction {
  my ($self, @args) = @_;
  return $self->_qc_runner(q{gc_fraction}, @args);
}

=head2 qc_gc_bias

run the gc_bias test

=cut

sub qc_gc_bias {
  my ($self, @args) = @_;
  return $self->_qc_runner(q{gc_bias}, @args);
}

################
# single method to actually run qc, which should be given the test from the called method requested
sub _qc_runner {
  my ($self, $qc_to_run, @args) = @_;
  my $required_job_completion = shift @args;

  my $afg = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::qc},{
    qc_to_run  => $qc_to_run,
  });

  my $arg_refs = {
    required_job_completion => $required_job_completion,
  };

  my @job_ids = $afg->run_qc($self->_autoqc_options_from_args($arg_refs, @args));
  return @job_ids;
}

sub _autoqc_options_from_args {
  my ($self, $arg_refs, @args) = @_;

  if (!defined $arg_refs) {
    $arg_refs = {};
  }
  if (@args) {
    my $options = pop @args;
    if (ref $options eq q[HASH] && exists $options->{autoqc_data} && ref $options->{autoqc_data} eq q[HASH] ) {
      foreach my $key (keys %{$options->{autoqc_data}}) {
	$arg_refs->{$key} = $options->{autoqc_data}->{$key};
      }
    }
  }
  return $arg_refs;
}


=head2 munge_summary_for_multiplex

munge the summary file produced, where it is a paired end multiplex run, which has had the hack done on it.
This shouldn't need to be bsubbed.

=cut

sub munge_summary_for_multiplex {
  my ($self, @args) = @_;

  if ( $self->no_munge_summary() ) {
    $self->log( q{user has determined not to munge the summary} );
    return ();
  }

  my $arg_refs = {};

  $arg_refs->{required_job_completion}  = shift @args;

  my $munge_summary = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::summary});
  $munge_summary->munge($arg_refs);

  return ();

}

=head2 check_cluster_counts

For each lane, job submitted which checks that cluster counts are what they are expected to be in various files

=cut

sub check_cluster_counts {
  my ( $self, @args ) = @_;

  my $arg_refs = {
    required_job_completion => shift @args,
  };

  my $cluster_count_checker = $self->new_with_cloned_attributes( q{npg_pipeline::archive::file::ClusterCounts} );
  return $cluster_count_checker->launch( $arg_refs );
}

=head2 create_schema_information

submits job which will create a schema file for each lane detailing info on what was run on that lane

=cut

sub create_schema_information {
  my ( $self, @args ) = @_;

  my $arg_refs = {
    required_job_completion => shift @args,
  };

  my $schema_information = $self->new_with_cloned_attributes( q{npg_pipeline::archive::file::SchemaInformation} );

  return $schema_information->launch_bsub_job( $arg_refs );

}

=head2 create_webcache_softlink

create a relative softlink to the webcache directory used for this analysis in the archive directory

=cut

sub create_webcache_softlink {
  my ( $self, @args ) = @_;

  return $self->new_with_cloned_attributes( q{npg_pipeline::archive::folder::WebCache} )->create_webcache_link( @args );
}

=head2 populate_archive_directory_from_mpsa

populate a stand-alone (not inside run folder) archive directory with relevant files from mpsa
perform file conversions (merge human split, split of old fastqs) if necessary

=cut

sub populate_archive_directory_from_mpsa {
  my ( $self, @args ) = @_;

  my $arg_refs = {
    required_job_completion => shift @args,
  };
  return $self->new_with_cloned_attributes( q{npg_pipeline::archive::mpsa_to_irods_migration_helper} )->launch_bsub_job( $arg_refs );
}

=head2 run_post_qc_review

performs post_qc_review step as a dependency of all preceeding post_qseq jobs

=cut

sub run_post_qc_review {
  my ( $self, @args ) = @_;

  my $arg_refs = {
    required_job_completion => shift @args,
  };
  return $self->new_with_cloned_attributes( q{npg_pipeline::archive::mpsa_to_irods_migration_helper} )->launch_bsub_job($arg_refs);
}

=head2 touch_completed_lane

launches a job which will touch a file in the archive directory to say that post_qseq has completed

=cut

sub touch_completed_lane {
  my ( $self, @args ) = @_;

  my $arg_refs = {
    required_job_completion => shift @args,
  };
  return $self->new_with_cloned_attributes( q{npg_pipeline::archive::file::generation::touchCompletedLane} )->launch_bsub_job($arg_refs);
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
