#############
# $Id: FilesPresent.pm 14173 2011-09-13 09:45:29Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2011-06-27
# Last Changed On: $Date: 2011-09-13 10:45:29 +0100 (Tue, 13 Sep 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-30.0/lib/npg_pipeline/pre_exec/FilesPresent.pm $

package npg_pipeline::pre_exec::FilesPresent;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 14173 $ =~ /(\d+)/mxs; $r; };

extends q{npg_pipeline::base};
with q{npg_pipeline::roles::business::pre_exec_strings};

=head1 NAME

npg_pipeline::pre_exec::FilesPresent

=head1 VERSION

$LastChangedRevision: 14173 $

=head1 SYNOPSIS

  my $ref_adapters = npg_pipeline::pre_exec::FilesPresent->new();
  $ref_adapters->check_existence();

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 check_existence

This is the only method, and it checks that the files provided can be found
If they can't, then the method croaks.

=cut

sub check_existence {
  my ( $self ) = @_;

  eval {
    # This does a check on the reference repository by default
    foreach my $file ( $self->all_files() ) {
      if ( ! -e $file ) {
        croak qq{unable to find file $file.\nTherefore, at least one lanes post_qseq has not finished};
      }
    }
    1;
  } or do {
    croak $EVAL_ERROR;
  };

  return 1;
}

has q{files} => (
  traits     => [ q{Array} ],
  isa         => q{ArrayRef[Str]},
  is          => q{ro},
  default    => sub { [] },
  handles    => {
    all_files    => q{elements},
    count_files  => q{count},
    has_no_files => q{is_empty},
  },
);

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2011 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
