# $Id: 10-pluggable_harold.t 14173 2011-09-13 09:45:29Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 17;
use Test::Deep;
use Test::Exception;
use lib qw{t};
use t::util;
use t::dbic_util;
use File::Slurp;

my $util = t::util->new();
$util->catch_email();

$ENV{DBIC_TRACE} = 0;

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
$ENV{OWNING_GROUP} = q{staff};

use_ok('npg_pipeline::pluggable::harold');

my $analysis_runfolder_path = $util->analysis_runfolder_path();
$ENV{TEST_USERAGENT} = $analysis_runfolder_path . q{/useragent};

my $schema = t::dbic_util->new->test_schema();

{
  my $harold;
  my $functions_in_order = [qw(
      create_archive_directory
      create_fastq
      create_fastqcheck
      create_gcfreq
      create_md5
    )];
  lives_ok {
    $harold = npg_pipeline::pluggable::harold->new({
      schema => q{harold},
      script_name => q{test},
      id_run => 1234,
      function_order => $functions_in_order,
      run_folder => q{123456_IL2_1234},
      analysis_path => q{Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto},
      runfolder_path => $analysis_runfolder_path,
    });
  } q{no croak on creation};
  isa_ok($harold, q{npg_pipeline::pluggable::harold}, q{$harold});
  is($harold->function_order(), $functions_in_order, q{$harold->function_order() set on creation});

}

$util->set_staging_analysis_area({with_latest_summary => 1});
write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );

{
  my $harold;
  lives_ok {
    $harold = npg_pipeline::pluggable::harold->new({
      schema => q{harold},
      script_name => q{test},
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      domain => q{test},
      conf_path => q{data/config_files},
    });
  } q{no croak on new creation};
  my @function_order;
  lives_ok {
    @function_order = @{$harold->_function_order()};
  } q{no croak setting function order};
  is_deeply(\@function_order, [], q{function order to be used would be base default});

  $harold->_clear_function_order;

  $harold->precreated_function_order(q{spam});
  lives_ok {
    @function_order = @{$harold->_function_order()};
  } q{no croak setting function order};
  is_deeply(\@function_order, [], q{function order to be used would be base default when a value for precreated_function_order is not an accepted one});

  $harold->_clear_function_order;

  $harold->precreated_function_order(q{archive_run});
  is($harold->precreated_function_order(), q{archive_run}, q{$harold->precreated_function_order() set});
  lives_ok {
    @function_order = @{$harold->_function_order()};
  } q{no croak setting function order};
  is_deeply(\@function_order, $harold->function_order_conf()->{post_qc_review}, q{function order to be used would be post_qc_review});

  my $parallelise_hash;
  lives_ok {
    $parallelise_hash = $harold->parallelise();
  } q{croak obtaining parallelise_hash};
}

{
  my $harold;
  lives_ok {
    $harold = npg_pipeline::pluggable::harold->new({
      schema => q{harold},
      script_name => q{test},
      id_run => 6588,
      lanes => [1..8],
      run_folder => q{123456_IL2_1234},
      runfolder_path => $analysis_runfolder_path,
      dbix_tracking_schema => $schema,
      no_bsub => 1,
      conf_path => $util->conf_path(),
      domain => q{test},
    });
  } q{no croak on creation};

  lives_ok {
    $harold->update_run_lane_analysis_in_progress();
  } q{run update_run_lane_analysis_in_progress ok - no dependencies};
  lives_ok {
    $harold->update_run_lane_analysis_complete({
      required_job_completion => q{-w'done(123) && done(321)'},
    });
  } q{run update_run_lane_analysis_complete ok - dependencies};
  
}
{
  my $harold = npg_pipeline::pluggable::harold->new({
      schema => q{harold},
      script_name => q{test},
      id_run => 6588,
      run_folder => q{123456_IL2_1234},
      no_bsub => 1,
      no_status_updates => 1,
  });
  my @jids = $harold->update_run_lane_analysis_complete();
  is( scalar @jids, 0, q{no jobs submitted, as no_status_updates true} );
}
1;