# $Id: 20-archive_file-split_phix.t 14173 2011-09-13 09:45:29Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 19;
use Test::Exception;
use lib qw{t};
use t::util;

use_ok('npg_pipeline::archive::file::split_phix');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();

my $analysis_runfolder_path = $util->analysis_runfolder_path();

{

  my $split_phix = npg_pipeline::archive::file::split_phix->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    repository => q{t/data/sequence},
  });
  isa_ok($split_phix, q{npg_pipeline::archive::file::split_phix}, q{$split_phix});

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
    array_string => q{[1-3,5,7-8]},
  };

  $util->set_rta_staging_analysis_area();

  $split_phix->_populate_directory_paths();
my $hex;
my $md5_object;
  my $bsub_command = $util->drop_temp_part_from_paths( $split_phix->_generate_bsub_command( $split_phix->lsb_jobindex(), $arg_refs ) );
  my $expected_cmd = q{bsub -q test -w'done(123) && done(321)' -J split_phix_1234_20090709-123456[1-3,5,7-8] -R 'select[mem>8000] rusage[mem=8000,nfs_12=8] span[hosts=1]' -M8000000 -n8 -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/log/split_phix_1234_20090709-123456.%I.%J.out -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' 't/bin/software/solexa/bin/fastq_srf_split -fastq1 /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`.fastq  -no_pileup -ref_name phix -ref_bwa t/data/sequence/references/PhiX/default/all/bwa/phix-illumina.fa -ref_fa t/data/sequence/references/PhiX/default/all/fasta/phix-illumina.fa -fork_alignment -id_run 1234 -position `echo $LSB_JOBINDEX` -output_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`.fastq /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_orig.fastq && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_nonphix.fastq /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`.fastq && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`.fastqcheck /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_orig.fastqcheck && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_nonphix.fastqcheck /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`.fastqcheck && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`.fastq.md5 /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_orig.fastq.md5 && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_nonphix.fastq.md5 /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`.fastq.md5'};
  is( $bsub_command, $expected_cmd, q{$split_phix->_generate_bsub_command( $split_phix->lsb_jobindex(), $arg_refs ) result ok});

  $arg_refs->{paired}++;
  $bsub_command = $util->drop_temp_part_from_paths( $split_phix->_generate_bsub_command( $split_phix->lsb_jobindex(), $arg_refs ) );
  $expected_cmd = q{bsub -q test -w'done(123) && done(321)' -J split_phix_1234_20090709-123456[1-3,5,7-8] -R 'select[mem>8000] rusage[mem=8000,nfs_12=8] span[hosts=1]' -M8000000 -n8 -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/log/split_phix_1234_20090709-123456.%I.%J.out -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' 't/bin/software/solexa/bin/fastq_srf_split -fastq1 /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1.fastq -fastq2 /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2.fastq  -no_pileup -ref_name phix -ref_bwa t/data/sequence/references/PhiX/default/all/bwa/phix-illumina.fa -ref_fa t/data/sequence/references/PhiX/default/all/fasta/phix-illumina.fa -fork_alignment -id_run 1234 -position `echo $LSB_JOBINDEX` -output_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1.fastq /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1_orig.fastq && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1_nonphix.fastq /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1.fastq && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1.fastqcheck /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1_orig.fastqcheck && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1_nonphix.fastqcheck /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1.fastqcheck && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1.fastq.md5 /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1_orig.fastq.md5 && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1_nonphix.fastq.md5 /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1.fastq.md5 && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2.fastq /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2_orig.fastq && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2_nonphix.fastq /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2.fastq && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2.fastqcheck /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2_orig.fastqcheck && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2_nonphix.fastqcheck /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2.fastqcheck && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2.fastq.md5 /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2_orig.fastq.md5 && mv /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2_nonphix.fastq.md5 /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2.fastq.md5'};
  is( $bsub_command, $expected_cmd, q{$split_phix->_generate_bsub_command( $split_phix->lsb_jobindex(), $arg_refs ) paired result ok});

  my $rename_command = $util->drop_temp_part_from_paths( $split_phix->_rename_file_commands( q{/tmp/QIsAJTewKA/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1.fastq} ) );
  $expected_cmd = q{mv /tmp/QIsAJTewKA/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1.fastq /tmp/QIsAJTewKA/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1_orig.fastq && mv /tmp/QIsAJTewKA/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1_nonphix.fastq /tmp/QIsAJTewKA/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1.fastq};
  is( $rename_command, $expected_cmd, q{rename fastq file name command});

}

{
  my $split_phix = npg_pipeline::archive::file::split_phix->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    repository => q{t/data/sequence},
  });
  isa_ok($split_phix, q{npg_pipeline::archive::file::split_phix}, q{$split_phix});

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  $util->set_rta_staging_analysis_area();

  my $cache = $split_phix->_lane_phix_splitting_cache();

  my @jids;
  foreach my $lane (1..8) {
    $cache->{$lane} = q{true};
  }
  lives_ok { @jids = $split_phix->split_run_lanes( $arg_refs ); } q{no croak running split};

  is(scalar@jids, 1, q{1 job id returned});
}

{
  my $split_phix = npg_pipeline::archive::file::split_phix->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    repository => q{t/data/sequence},
  });
  isa_ok($split_phix, q{npg_pipeline::archive::file::split_phix}, q{$split_phix});

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  $util->set_rta_staging_analysis_area();

  my $cache = $split_phix->_lane_phix_splitting_cache();

  my @jids;
  foreach my $lane (1..8) {
    $cache->{$lane} = q{false};
  }
  lives_ok { @jids = $split_phix->split_run_lanes( $arg_refs ); } q{no croak running split};

  is(scalar@jids, 0, q{0 job ids returned});
}

{
  $ENV{TEST_BIN} = undef;
  $ENV{TEST_RUN_OBJECT} = npg::api::run->new({
    util => $util->apiutil(),
    id_run => 1234,
  });
  
  my $split_phix = npg_pipeline::archive::file::split_phix->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123457},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    repository => q{t/data/sequence},
  });
  $split_phix->_populate_directory_paths();

  is($split_phix->_is_lane_spiked_phix(1), q{false}, q{lane 1 does not contain spiked phix});
  is($split_phix->_is_lane_spiked_phix(2), q{true},  q{lane 2 does not contain spiked phix});
}

{
$ENV{TEST_BIN} = q{t/bin/};
  my $split_phix = npg_pipeline::archive::file::split_phix->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    lane => 4,
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    repository => q{t/data/sequence},
  });
  isa_ok($split_phix, q{npg_pipeline::archive::file::split_phix}, q{$split_phix});

  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};

  $util->set_rta_staging_analysis_area();

  $split_phix->_populate_directory_paths();

  my @jids;
  foreach my $lane (1..8) {
    $split_phix->_lane_phix_splitting_cache()->{$lane} = q{true};
  }
  eval { @jids = $split_phix->split_run_lanes($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak running split});

  is(scalar@jids, 1, q{1 job id returned});
}

{
  my $split_phix = npg_pipeline::archive::file::split_phix->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    lane => 4,
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    repository => q{t/data/sequence},
  });
  isa_ok($split_phix, q{npg_pipeline::archive::file::split_phix}, q{$split_phix});

  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};
  $arg_refs->{multiplexed}              = 4;

  $util->set_rta_staging_analysis_area();

  $split_phix->_populate_directory_paths();

  my @jids;
  foreach my $lane (1..8) {
    $split_phix->_lane_phix_splitting_cache()->{$lane} = q{true};
  }
  eval { @jids = $split_phix->split_run_lanes($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak running split when there is a multiplexed run});

  is(scalar@jids, 1, q{1 job id returned});
}

1;
