# $Id: 25-analysis-FixConfigFiles.t 14173 2011-09-13 09:45:29Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';#tests => ;
use Test::Exception;
use lib qw{t};
use t::util;
use File::Slurp;

BEGIN {
  use_ok( q{npg_pipeline::analysis::FixConfigFiles} );
}
my $util = t::util->new();

my $runfolder_path = $util->hiseq_runfolder_path();
my $intensity_path = $runfolder_path . q{/Data/Intensities};

{

  my $fix_config_files;
  lives_ok {
    $fix_config_files = npg_pipeline::analysis::FixConfigFiles->new( {
      id_run => 1234,
      runfolder_name_ok => 0,
      last_cycle_numbers_ok => 1,
    } );
  } q{new object created ok};

  isa_ok( $fix_config_files, q{npg_pipeline::analysis::FixConfigFiles}, q{$fix_config_files} );

  throws_ok {
    $fix_config_files->run();
  } qr{problem[ ]with[ ]runfolder_name[ ]or[ ]last_cycle_numbers}, q{runfolder_name is not ok, throws error};

  $fix_config_files = npg_pipeline::analysis::FixConfigFiles->new( {
    id_run => 1234,
    runfolder_name_ok => 1,
    last_cycle_numbers_ok => 0,
  } );
  throws_ok {
    $fix_config_files->run();
  } qr{problem[ ]with[ ]runfolder_name[ ]or[ ]last_cycle_numbers}, q{last_cycle_numbers are not ok, throws error};

  $fix_config_files = npg_pipeline::analysis::FixConfigFiles->new( {
    id_run => 1234,
    runfolder_path => $runfolder_path,
    no_npg_check => 1,
    last_cycle_numbers_ok => 1,
    intensity_path => $intensity_path,
  } );

  lives_ok {
    $fix_config_files->basecalls_xml();
  } q{BaseCalls/config.xml read in ok};

  lives_ok {
    $fix_config_files->run();
  } q{run ok};

  my $changed_intensities_file = read_file ( $intensity_path . q{/config.xml} );
  my $changed_basecalls_file   = read_file ( $intensity_path . q{/BaseCalls/config.xml} );

  my $expected_intensities_file = read_file ( q{t/data/fix_configs/fixed_intensities_config.xml} );
  my $expected_basecalls_file   = read_file ( q{t/data/fix_configs/fixed_basecalls_config.xml} );

  is( $changed_intensities_file, $expected_intensities_file, q{intensities file has been changed correctly} );
  is( $changed_basecalls_file,   $expected_basecalls_file,   q{basecalls file has been changed correctly} );

}

{
  my $fix_config_files = npg_pipeline::analysis::FixConfigFiles->new( {
    id_run => 1234,
    runfolder_path => $runfolder_path,
    intensity_path => $intensity_path,
  } );

  ok( $fix_config_files->last_cycle_numbers_ok(), q{All last cycle numbers are the same} );

}
{
  $util->hiseq_runfolder_path( 1 );
  my $fix_config_files = npg_pipeline::analysis::FixConfigFiles->new( {
    id_run => 1234,
    runfolder_path => $runfolder_path,
    intensity_path => $intensity_path,
  } );

  ok( ! $fix_config_files->last_cycle_numbers_ok(), q{Not all last cycle numbers are the same} );

}

1;