# $Id: 25-analysis-SecondBaseCall.t 14173 2011-09-13 09:45:29Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';#tests => ;
use Test::Exception;
use lib qw{t};
use t::util;

BEGIN {
  use_ok( q{npg_pipeline::analysis::SecondBaseCall} );
}

my $util = t::util->new();

my $paths = $util->set_rta_staging_analysis_area();

{

  my $sbc;
  lives_ok {
    $sbc = npg_pipeline::analysis::SecondBaseCall->new( {
      id_run => 1234,
      recalibrated_path => $paths->{recalibrated_path},
      conf_path => $util->cwd() . q{/data/config_files},
      domain => q{test},
      no_bsub => 1,
      timestamp => q{20101118-091411},
      lanes => [ 1..8 ],
      instrument_type => q{HiSeq},
    } );
  } q{created object ok};
  isa_ok( $sbc, q{npg_pipeline::analysis::SecondBaseCall}, q{$sbc} );

  $sbc->make_log_dir( $sbc->intensity_path() . q{/BaseCalls} );

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
    read_number => 1,
  };

  my @jids;
  lives_ok {
    @jids = $sbc->submit_to_lsf( $arg_refs );
  } q{submit_to_lsf runs ok};

  is( scalar @jids, 1, q{1 job id obtained} );

  my $expected_bsub_command = q{bsub -q test -J 'second_basecall_1234_20101118-091411[1-8]' -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/SecondCall/log/second_basecall_1234_20101118-091411.%I.%J.out 't/bin/software/solexa/src/new-pipeline-dev/pb_calibration/branches/current/bin/pb_second_basecall -bustard_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA -runfolder_dir /nfs/sf45/IL2/analysis/123456_IL2_1234 -run 1234 -lane `echo $LSB_JOBINDEX` -read 1 -cstart 1 -scl'};

  is( $util->drop_temp_part_from_paths( $sbc->_generate_bsub_command( $arg_refs ) ), $expected_bsub_command, q{generated bsub command is correct} );

}
{

  my $sbc;
  lives_ok {
    $sbc = npg_pipeline::analysis::SecondBaseCall->new( {
      id_run => 1234,
      recalibrated_path => $paths->{recalibrated_path},
      conf_path => $util->cwd() . q{/data/config_files},
      domain => q{test},
      no_bsub => 1,
      timestamp => q{20101118-091411},
      lanes => [ 1..8 ],
      instrument_type => q{HK},
      is_paired_read => 1,
    } );
  } q{created object ok};

  $sbc->_push_read2_cycle_range( 38 );

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  my @jids;
  lives_ok {
    @jids = $sbc->submit_to_lsf( $arg_refs );
  } q{submit_to_lsf runs ok};

  is( scalar @jids, 2, q{2 job ids obtained - paired read} );

  $arg_refs->{read_number} = 2;

  my $expected_bsub_command = q{bsub -q test -J 'second_basecall_1234_20101118-091411[1-8]' -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/SecondCall/log/second_basecall_1234_20101118-091411.%I.%J.out 't/bin/software/solexa/src/new-pipeline-dev/pb_calibration/branches/current/bin/pb_second_basecall -bustard_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA -runfolder_dir /nfs/sf45/IL2/analysis/123456_IL2_1234 -run 1234 -lane `echo $LSB_JOBINDEX` -read 2 -cstart 38'};

  is( $util->drop_temp_part_from_paths( $sbc->_generate_bsub_command( $arg_refs ) ), $expected_bsub_command, q{generated bsub command is correct} );

}
{

  my $sbc;
  lives_ok {
    $sbc = npg_pipeline::analysis::SecondBaseCall->new( {
      id_run => 1234,
      recalibrated_path => $paths->{recalibrated_path},
      conf_path => $util->cwd() . q{/data/config_files},
      domain => q{test},
      no_bsub => 1,
      timestamp => q{20101118-091411},
      lanes => [ 1..8 ],
      instrument_type => q{HK},
      is_indexed => 1,
      is_paired_read => 1,
    } );
  } q{created object ok};

  $sbc->_push_read2_cycle_range( 84 );
  $sbc->_push_indexing_cycle_range( 77 );

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  my @jids;
  lives_ok {
    @jids = $sbc->submit_to_lsf( $arg_refs );
  } q{submit_to_lsf runs ok};

  is( scalar @jids, 3, q{3 job ids obtained - paired and indexed read} );

  $arg_refs->{read_number} = 2;

  my $expected_bsub_command = q{bsub -q test -J 'second_basecall_1234_20101118-091411[1-8]' -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/SecondCall/log/second_basecall_1234_20101118-091411.%I.%J.out 't/bin/software/solexa/src/new-pipeline-dev/pb_calibration/branches/current/bin/pb_second_basecall -bustard_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA -runfolder_dir /nfs/sf45/IL2/analysis/123456_IL2_1234 -run 1234 -lane `echo $LSB_JOBINDEX` -read 2 -cstart 77'};

  is( $util->drop_temp_part_from_paths( $sbc->_generate_bsub_command( $arg_refs ) ), $expected_bsub_command, q{generated bsub command is correct - indexing read} );

}
{

  my $sbc;
  lives_ok {
    $sbc = npg_pipeline::analysis::SecondBaseCall->new( {
      id_run => 1234,
      recalibrated_path => $paths->{recalibrated_path},
      conf_path => $util->cwd() . q{/data/config_files},
      domain => q{test},
      no_bsub => 1,
      timestamp => q{20101118-091411},
      lanes => [ 1..8 ],
      instrument_type => q{HK},
      is_indexed => 1,
    } );
  } q{created object ok};

  $sbc->_push_indexing_cycle_range( 109 );

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  my @jids;
  lives_ok {
    @jids = $sbc->submit_to_lsf( $arg_refs );
  } q{submit_to_lsf runs ok};

  is( scalar @jids, 2, q{2 job ids obtained - indexed read} );

  $arg_refs->{read_number} = 2;

  my $expected_bsub_command = q{bsub -q test -J 'second_basecall_1234_20101118-091411[1-8]' -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/SecondCall/log/second_basecall_1234_20101118-091411.%I.%J.out 't/bin/software/solexa/src/new-pipeline-dev/pb_calibration/branches/current/bin/pb_second_basecall -bustard_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA -runfolder_dir /nfs/sf45/IL2/analysis/123456_IL2_1234 -run 1234 -lane `echo $LSB_JOBINDEX` -read 2 -cstart 109'};

  is( $util->drop_temp_part_from_paths( $sbc->_generate_bsub_command( $arg_refs ) ), $expected_bsub_command, q{generated bsub command is correct - indexing read} );

}
1;