# $Id: 30-run_lane_status.t 14173 2011-09-13 09:45:29Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 16;
use Test::Exception;
use lib qw{t};
use t::dbic_util;
use t::util;
use DateTime;

my $util = t::util->new();
my $conf_path = $util->conf_path();

BEGIN {
  use_ok(q{npg_pipeline::run::status::UpdateLane});
}
my $schema = t::dbic_util->new->test_schema();

{

  my $paths = $util->set_rta_staging_analysis_area(0,0,6588);
  my $updater;
  lives_ok {
    $updater = npg_pipeline::run::status::UpdateLane->new({
      dbix_tracking_schema => $schema,
      id_run => 6588,
      lane => 1,
      status => 'analysis in progress',
      runfolder_path => $paths->{runfolder_path},
    });
  } q{object created ok};

  is( $updater->dbix_tracking_schema(), $schema, q{have the test dbix schema} );
  isa_ok( $updater, q{npg_pipeline::run::status::UpdateLane}, q{$updater} );

  lives_ok {
    $updater->update();
  } q{run lane status updated ok};

}

{

  my $paths = $util->set_rta_staging_analysis_area(0,0,1);
  my $updater;
  lives_ok {
    $updater = npg_pipeline::run::status::UpdateLane->new({
      dbix_tracking_schema => $schema,
      id_run => 1,
      lane => 7,
      status => 'analysis complete',
      runfolder_path => $paths->{runfolder_path},
    });
  } q{object created ok};

  isa_ok( $updater, q{npg_pipeline::run::status::UpdateLane}, q{$updater} );

  lives_ok {
    $updater->update();
  } q{run lane status updated ok};

}
{

  my $paths = $util->set_rta_staging_analysis_area(0,0,1);
  my $updater;
  lives_ok {
    $updater = npg_pipeline::run::status::UpdateLane->new({
      dbix_tracking_schema => $schema,
      id_run => 1,
      position => 7,
      status => 'analysis complete',
      runfolder_path => $paths->{runfolder_path},
    });
  } q{object created ok};

  lives_ok {
    $updater->update();
  } q{run lane status updated ok};

}
{

  my $paths = $util->set_rta_staging_analysis_area(0,0,1);
  my $updater;
  lives_ok {
    $updater = npg_pipeline::run::status::UpdateLane->new({
      dbix_tracking_schema => $schema,
      id_run => 1,
      status => 'analysis complete',
      runfolder_path => $paths->{runfolder_path},
    });
  } q{object created ok};

  throws_ok {
    $updater->update();
  } qr{No[ ]lane/position[ ]provided}, q{run lane status update failed as no position provided};

}

{

  my $paths = $util->set_rta_staging_analysis_area();
  my $updater;
  lives_ok {
    $updater = npg_pipeline::run::status::UpdateLane->new({
      dbix_tracking_schema => $schema,
      id_run => 1234,
      lanes => [7],
      status => 'analysis complete',
      no_bsub => 1,
      conf_path => $conf_path,
      domain => q{test},
      runfolder_path => $paths->{runfolder_path},
      timestamp => q{20110905-151204},
    });
  } q{object created ok};

  my @job_ids = $updater->launch_bsub_jobs();
  is( scalar @job_ids, 1, q{only 1 lane, one job} );
  my $expected_bsub_command = qq{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_connect_dbix_tracking'  -J update_run_lane_status_1234_7_20110905-151204 -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/log/update_run_lane_status_1234_7_20110905-151204.%I.%J.out 't/bin/software/solexa/bin/npg_pipeline_update_lane_status --archive_path "/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive" --basecall_path "/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls" --bustard_path "/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA" --conf_path "data/config_files" --data_path "/nfs/sf45/IL2/analysis/123456_IL2_1234/Data" --domain "test" --id_run "1234" --intensity_path "/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities" --internal_script_bin "t/bin/software/solexa/bin" --lsf_queue "test" --no_bsub  --pb_cal_path "/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/PB_cal" --recalibrated_path "/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA" --runfolder_path "/nfs/sf45/IL2/analysis/123456_IL2_1234" --status "analysis complete" --position "7"'};
  is ( $util->drop_temp_part_from_paths( $updater->_generate_bsub_command({
    lane => 7,
    required_job_completion => q{},
  }) ), $expected_bsub_command, q{generated bsub command is correct} );


}

{
  lives_ok {
    npg_pipeline::run::status::UpdateLane->new({
      id_run => 1234,
      lanes => [7],
      status => 'analysis complete',
    })->dbix_tracking_schema();
  } q{connection made - note, do not do anything with this connection in a test};
}

1;
