# $Id: 35-checking-cluster_counts.t 14173 2011-09-13 09:45:29Z ajb $

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 37;
use Test::Exception::LessClever;
use lib qw{t};
use t::util;
use DateTime;

BEGIN {
  use_ok( q{npg_pipeline::archive::file::ClusterCounts} );
}

my $util = t::util->new({});
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();


$util->create_multiplex_analysis();
my $analysis_runfolder_path = $util->analysis_runfolder_path();
my $bustard_path = $util->standard_analysis_bustard_path();
qx{cp t/data/summary_files/BustardSummary_mp.xml $bustard_path/BustardSummary.xml};
my $recalibrated_path = $util->standard_analysis_recalibrated_path();
my $archive_path = $recalibrated_path . q{/archive};

{
  my $object;
  lives_ok {
    $object = npg_pipeline::archive::file::ClusterCounts->new({
      run_folder => q{123456_IL2_1234},
      id_run => 1234,
      timestamp => q{20100907-142417},
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};

  isa_ok( $object, q{npg_pipeline::archive::file::ClusterCounts}, q{$object} );

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
    array_string => q{[1-8]},
  };

  my $bsub_command = $util->drop_temp_part_from_paths( qq{bsub -q test -w'done(123) && done(321)' -J 'npg_pipeline_check_file_cluster_count_1234_20100907-142417[1-8]' -o $archive_path/log/npg_pipeline_check_file_cluster_count_1234_20100907-142417.} . q{%I.%J.out 't/bin/software/solexa/bin/npg_pipeline_check_file_cluster_count --id_run=1234 --position=`echo $LSB_JOBINDEX` --archive_path=} . qq{$archive_path --bustard_path=$bustard_path'} );
  is( $util->drop_temp_part_from_paths( $object->_generate_bsub_command( $arg_refs ) ), $bsub_command, q{generated bsub command is correct} );

  my @jids = $object->launch( $arg_refs );
  is( scalar @jids, 1, q{1 job id returned} );
}

{
  my $object;
  lives_ok{
    $object = npg_pipeline::archive::file::ClusterCounts->new({
      id_run => 1234,
      position => 5,
      bustard_path => $bustard_path,
      archive_path => $archive_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};

  throws_ok {
    $object->run_cluster_count_check();
  } qr{inconsistent[ ]raw[ ]counts[ ]between[ ]reads}, q{$object->run() throws ok - inconsistent raw};

}
{
  my $object;
  lives_ok{
    $object = npg_pipeline::archive::file::ClusterCounts->new({
      id_run => 1234,
      position => 6,
      bustard_path => $bustard_path,
      archive_path => $archive_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};

  throws_ok {
    $object->run_cluster_count_check();
  } qr{inconsistent[ ]pf[ ]counts[ ]between[ ]reads}, q{$object->run() throws ok - inconsistent pf};

}

{
  my $object;
  lives_ok{
    $object = npg_pipeline::archive::file::ClusterCounts->new({
      id_run => 1234,
      position => 1,
      bustard_path => $bustard_path,
      archive_path => $archive_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};

  qx{mkdir $archive_path/qc};
  qx{cp t/data/bam_flagstats/1234_1_bam_flagstats.json $archive_path/qc };

  lives_ok {
    $object->run_cluster_count_check();
  } qr{check returns ok};
}
{
  my $object;
  lives_ok{
    $object = npg_pipeline::archive::file::ClusterCounts->new({
      id_run => 1234,
      position => 2,
      bustard_path => $bustard_path,
      archive_path => $archive_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};

#  qx{cp t/data/shortened_fastqs/1234_2_1.fastq $archive_path/1234_2_1.fastq};
#  throws_ok {
#    $object->run_cluster_count_check();
#  } qr{1234_2_1[.]fastq[ ]does[ ]not[ ]contain[ ]the[ ]correct[ ]number[ ]of[ ]pf[ ]clusters}, q{wrong count of reads in fastq};

#  qx{rm $archive_path/1234_2_1.fastq};
  qx{cp t/data/shortened_fastqs/1234_2_1.fastq $archive_path/1234_2_1_human.fastq};
  qx{cp t/data/shortened_fastqs/1234_2_1.fastq $archive_path/1234_2_1_nonhuman.fastq};
  qx{cp t/data/shortened_fastqs/1234_2_1.fastq $archive_path/1234_2_2_human.fastq};
  qx{cp t/data/shortened_fastqs/1234_2_1.fastq $archive_path/1234_2_2_nonhuman.fastq};
  qx{cp t/data/shortened_fastqs/1234_2_1.fastq $archive_path/1234_2_t_human.fastq};
  qx{cp t/data/shortened_fastqs/1234_2_1.fastq $archive_path/1234_2_t_nonhuman.fastq};
#  throws_ok {
#    $object->run_cluster_count_check();
#  } qr{split[ ]read_one[ ]does[ ]not[ ]contain[ ]the[ ]correct[ ]number[ ]of[ ]pf[ ]clusters\n\tExpected:[ ]15\n\tActual:[ ]40}, q{wrong count of reads in fastq};

  qx{echo 20 > $archive_path/1234_2_1.fastqcheck};
  throws_ok {
    $object->run_cluster_count_check();
  } qr{At least one of the cluster count checks failed}, q{wrong pf read count in fastqcheck file};

}

{
  qx{echo 15 > $archive_path/1234_4_1.fastqcheck};
  qx{echo 15 > $archive_path/1234_4_2.fastqcheck};
  qx{echo 15 > $archive_path/1234_4_t.fastqcheck};
  qx{cp t/data/shortened_fastqs/1234_4_1.fastq $archive_path/1234_4_1.fastq};
  qx{cp t/data/shortened_fastqs/1234_4_1.fastq $archive_path/1234_4_2.fastq};
  qx{cp t/data/shortened_fastqs/1234_4_1.fastq $archive_path/1234_4_t.fastq};
  qx{cp t/data/shortened_fastqs/1234_4_1_human.fastq $archive_path/1234_4_1_human.fastq};
  qx{cp t/data/shortened_fastqs/1234_4_1_nonhuman.fastq $archive_path/1234_4_1_nonhuman.fastq};
  my $object;
  lives_ok{
    $object = npg_pipeline::archive::file::ClusterCounts->new({
      id_run => 1234,
      position => 4,
      bustard_path => $bustard_path,
      archive_path => $archive_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};

  qx{mkdir $archive_path/qc};
  qx{cp t/data/bam_flagstats/1234_4_bam_flagstats.json $archive_path/qc };
  lives_ok {
    $object->run_cluster_count_check();
  } q{$object->run() ok};

  is( $object->_bustard_pf_cluster_count(),  15, q{correct pf_cluster_count obtained from BustardSummary.xml}  );
  is( $object->_bustard_raw_cluster_count(), 30, q{correct raw_cluster_count obtained from BustardSummary.xml} );

}

{
  lives_ok {
    qx{perl bin/npg_pipeline_check_file_cluster_count --conf_path $conf_path --domain live --id_run 1234 --position 4 --bustard_path $bustard_path --archive_path $archive_path}
  } q{script runs ok};
  ok( ! $CHILD_ERROR, q{script completed ok - no croak} );
  lives_ok {
    qx{perl bin/npg_pipeline_check_file_cluster_count --conf_path $conf_path --domain live --id_run=1234 --position=2 --bustard_path=$bustard_path --archive_path=$archive_path}
  } q{script runs ok};
  ok( $CHILD_ERROR, qq{script croaked - $CHILD_ERROR} );
}

{
  my $object;
  lives_ok{
    $object = npg_pipeline::archive::file::ClusterCounts->new({
      id_run => 1234,
      position => 3,
      bustard_path => $bustard_path,
      archive_path => $archive_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};

  qx{echo 15 > $archive_path/1234_3_1_phix.fastqcheck};
  qx{echo 15 > $archive_path/1234_3_2_phix.fastqcheck};
  qx{echo 15 > $archive_path/1234_3_t_phix.fastqcheck};
  my @spiked_phix_fastcheck_files = $object->_spiked_phix_fqnames(1);
  is(scalar @spiked_phix_fastcheck_files, 3, 'correct number of phix fastqcheck files');
  is($object->_check_spiked_phix_cluster_count(1), 15, 'correct cluster count in fastqcheck files');

  qx{echo 25 > $archive_path/1234_3_t_phix.fastqcheck};
  throws_ok {$object->_check_spiked_phix_cluster_count(1)} qr{Cluster count in spiked phix file are different}, 'different cluster count in fastqcheck files';
  
  qx{cp t/data/shortened_fastqs/1234_4_1.fastq $archive_path/1234_3_1_phix.fastq};
  qx{cp t/data/shortened_fastqs/1234_4_1.fastq $archive_path/1234_3_2_phix.fastq};
  qx{cp t/data/shortened_fastqs/1234_4_1.fastq $archive_path/1234_3_t_phix.fastq}; 
  @spiked_phix_fastcheck_files = $object->_spiked_phix_fqnames();
  is(scalar @spiked_phix_fastcheck_files, 3, 'correct number of phix fastq files');
  is($object->_check_spiked_phix_cluster_count(), 15, 'correct cluster count in fastqcheck files');
  
  qx{head -12 t/data/shortened_fastqs/1234_4_1.fastq > $archive_path/1234_3_1_phix.fastq};
  throws_ok {$object->_check_spiked_phix_cluster_count()} qr{Cluster count in spiked phix file are different}, 'different cluster count in fastq files';
}


{
  my $object;
  lives_ok{
    $object = npg_pipeline::archive::file::ClusterCounts->new({
      id_run => 1234,
      position => 3,
      bustard_path => $bustard_path,
      archive_path => $archive_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};

  qx{echo 2 > $archive_path/1234_3_1_phix.fastqcheck};
  qx{echo 2 > $archive_path/1234_3_2_phix.fastqcheck};
  qx{echo 2 > $archive_path/1234_3_t_phix.fastqcheck};
  
  qx{echo 13 > $archive_path/1234_3_1.fastqcheck};
  qx{echo 13 > $archive_path/1234_3_2.fastqcheck};
  qx{echo 13 > $archive_path/1234_3_t.fastqcheck};

  ok( $object->_check_fastqcheck(), 'all fastqcheck for lane 3');; 

  qx{echo 1 > $archive_path/1234_3_1_phix.fastqcheck};
  qx{echo 1 > $archive_path/1234_3_2_phix.fastqcheck};
  qx{echo 1 > $archive_path/1234_3_t_phix.fastqcheck};
  
  throws_ok {$object->_check_fastqcheck()} qr{fastqcheck does not contain the correct number of pf clusters}, 'all fastqcheck file for lane 3 failed';
  
  ok( !$object->_bam_cluster_count_total({}), 'no bam cluster count total returned');
  
  qx{mkdir $archive_path/qc};
  qx{cp t/data/bam_flagstats/1234_3_bam_flagstats.json $archive_path/qc };
    
  is( $object->_bam_cluster_count_total({}), 16, 'correct bam cluster count total');
  
  my $is_indexed = 1;
  qx{mkdir -p $archive_path/lane3/qc};
  qx{cp t/data/bam_flagstats/1234_3_bam_flagstats.json $archive_path/lane3/qc/1234_3#0_bam_flagstats.json};
  qx{cp t/data/bam_flagstats/1234_3_bam_flagstats.json $archive_path/lane3/qc/1234_3#1_bam_flagstats.json};
  is( $object->_bam_cluster_count_total( {plex=>$is_indexed} ), 32, 'correct bam cluster count total for plexes');
  
  qx{cp t/data/bam_flagstats/1234_3_phix_bam_flagstats.json $archive_path/qc};

  my $spiked_phix_only = 1;
  is( $object->_bam_cluster_count_total( {spiked_phix_only => $spiked_phix_only} ), 14, 'correct bam cluster count for phix');
  
  ok($object->_check_bam_cluster_count(), 'Cluster count in bam files as expected');
  
  qx{rm $archive_path/qc/1234_3_phix_bam_flagstats.json};
  throws_ok {$object->_check_bam_cluster_count()}  qr{Cluster count in bam files not as expected}, 'Cluster count in bam files not as expected';
  
}
1;
