# $Id: 40-Checker.t 14173 2011-09-13 09:45:29Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';#tests => ;
use Test::Exception;
use lib qw{t};
use t::util;
use File::Slurp;

BEGIN {
  use_ok( q{npg_pipeline::Checker} );
}

my $text = q{};
my $output = \$text;

my $run_command_method = sub {
  my ( $command, $command_name ) = @_;
  return ${ $output };
};

{
  my $checker_object;
  lives_ok {
    $checker_object = npg_pipeline::Checker->new(
      run_command => $run_command_method,
    );
  } q{checker_object created ok};

  isa_ok( $checker_object, q{npg_pipeline::Checker}, q{$checker_object} );

  is( $checker_object->run_command(), $run_command_method, q{test run command method used} );

  lives_ok {
    $checker_object->check();
  } q{runs ok when no runs stuck};

  $checker_object->clear_stuck_runs();

  $text = read_file( q{t/data/checker_files/runs_stuck.txt} );
  my $stuck_runs = [
    {
      'id_run' => '5737',
      'status' => 'archival in progress',
    },
    {
      'id_run' => '5768',
      'status' => 'secondary analysis in progress',
    },
    {
      'id_run' => '5769',
      'status' => 'secondary analysis in progress',
    },
    {
      'id_run' => '5784',
      'status' => 'secondary analysis in progress',
    },
  ];
  is_deeply( $checker_object->stuck_runs(), $stuck_runs, q{stuck_runs has produced the correct information} );

  $text = read_file( q{t/data/checker_files/stuck_jobs_per_run.txt} );
  my $stuck_jobs_per_run = [ qq{855965  srpipe  PEND  srpipeline sf-1-1-04      -        npg_qc_save_files.pl_5737_20110204-170604 Feb  4 17:06 Dependency condition invalid or never satisfied;},
    qq{855966  srpipe  PEND  srpipeline sf-1-1-04      -        npg_qc_illumina_analysis_loader_5737_20110204-170604 Feb  4 17:06 Dependency condition invalid or never satisfied;},
    qq{855967  srpipe  PEND  srpipeline sf-1-1-04      -        npg_qc_autoqc_data.pl_5737_20110204-170604 Feb  4 17:06 Dependency condition invalid or never satisfied;}
  ];

  my @results = $checker_object->_stuck_jobs_per_run( 5737 );
  is_deeply( \@results, $stuck_jobs_per_run, q{stuck_jobs_per_run returned correctly} );

  $text = read_file( q{t/data/checker_files/get_job_dependencies.txt} );
  my $job_deps = [ 858395, 858396, 858397, 858398, 858399, 858400, 858401, 858402 ];
  @results = $checker_object->_get_job_dependencies( 858403 );
  is_deeply( \@results, $job_deps, q{job_dependencies returned correctly} ); 

  $text = read_file( q{t/data/checker_files/assess_job_dependency.txt} );
  lives_ok {
    $checker_object->_assess_job_dependency( 858395 );
  } q{assess_job_dependency runs as expected};
}

$run_command_method = sub {
  my ( $command, $command_name ) = @_;
  
  my $out = $command_name eq q{'get stuck_runs'}  ? read_file( q{t/data/checker_files/runs_stuck.txt} )
          : $command_name =~ /run_information/xms ? read_file( q{t/data/checker_files/stuck_jobs_per_run.txt} )
          : $command_name =~ /bjobs/xms           ? read_file( q{t/data/checker_files/get_job_dependencies.txt} )
          :                                         read_file( q{t/data/checker_files/assess_job_dependency.txt} )
          ;
  
  return $out;
};
{
  my $checker_object;
  lives_ok {
    $checker_object = npg_pipeline::Checker->new(
      run_command => $run_command_method,
    );
  } q{checker_object created ok};
  is( $checker_object->run_command(), $run_command_method, q{new test run command method used} );

  lives_ok {
    $checker_object->check();
  } q{runs ok when runs stuck};
}

{
  my $checker_object = npg_pipeline::Checker->new();
  my $output = $checker_object->run_command()->( q{echo hello}, q{echo} );
  is( $output, qq{hello\n}, q{run_command ran correctly} );
  lives_ok {
    $checker_object->run_command()->( q{foo hello}, q{foo} );
  } q{command which fails throws correctly, but eval traps error};
}

1;