# $Id: 50-harold_analysis_runner.t 14173 2011-09-13 09:45:29Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 8;
use Test::Exception;
use lib qw{t};
use t::util;

BEGIN {
  use_ok('npg_pipeline::daemons::harold_analysis_runner');
}

my $util = t::util->new();
my $conf_path = $util->conf_path();

my $temp_directory = $util->temp_directory();

$ENV{TEST_BIN} = q{t/bin};
$ENV{TEST_LOG} = $util->temp_directory();
`rm -rf $temp_directory/software/solexa/logs; mkdir -p $temp_directory/software/solexa/logs`;

{
  my $runner;
  lives_ok { $runner = npg_pipeline::daemons::harold_analysis_runner->new({
    conf_path => $conf_path,
    domain => q{test},
  }); } q{object creation ok};
  isa_ok($runner, q{npg_pipeline::daemons::harold_analysis_runner}, q{$runner});
  $runner->_set_api_util($util->apiutil());
  lives_ok { $runner->run(); } q{no croak on $runner->run()};
  is($runner->_generate_command( {
    id_run => 1234,
    script => q{npg_pipeline_analyse_RTA},
    options => q{},
    inst_model => q{HiSeq},
  } ), q{source t/bin/software/oracle/etc/profile.oracle ; source t/bin/software/lsf/conf/profile.lsf ; t/bin/software/solexa/bin/npg_pipeline_analyse_RTA --verbose --id_run 1234 --instrument_model HiSeq --no_secondcall }, q{generated command is correct});
}
{
  my $runner = npg_pipeline::daemons::harold_analysis_runner->new({
    conf_path => $conf_path,
    domain => q{test},
  });
  $runner->_set_api_util($util->apiutil());
  lives_ok {
    my $count = 0;
    while ($count < 2) {
      eval {
        $runner->run();
        1;
      } or do {
        carp q{Error running harold_analysis_runner: } . $EVAL_ERROR . q{ - Going to sleep};
      };
      sleep 2;
      $count++;
    }
  } q{no croak running through twice - potentially as a daemon process};

}

{
  my $runner;
  lives_ok { $runner = npg_pipeline::daemons::harold_analysis_runner->new( {
    conf_path => $conf_path,
    domain => q{test},
    _is_rta => 1,
  } ); } q{object creation ok};

  $runner->_set_api_util( $util->apiutil() );
  lives_ok { $runner->run(); } q{no croak on $runner->run()};
}


1;